/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */

package com.illuminated.encc;

import java.util.*;
import java.io.*;

public class Config {

	/**
	 * Creates a new config object with the default properties.
	 */
	public Config() {
	}

	/**
	 * Looks in $HOME/.encc for a properties file
	 * and tries to set the configuration from it.
	 */
	public void load() {
		Properties p;
		Enumeration en;

		try {
			p = new Properties();
			p.load(new FileInputStream(getFile()));
		} catch (IOException e) {
			setDefaults();
			return;
		}

		encIn = p.getProperty("encIn", "Windows");
		encOut= p.getProperty("encOut", "UTF-8");
		inputDir = p.getProperty("inputDir");
		outputDir = p.getProperty("outputDir");
		regexIn = p.getProperty("regexIn");
		regexOut = p.getProperty("regexOut");
		newFile = p.getProperty("newFile");

		en = p.propertyNames();
		while (en.hasMoreElements()) {
			String n = (String)en.nextElement();

			if (n.startsWith("addSubExtension.")) {
				addSubExtension.put(n.substring("addSubExtension.".length()), p.getProperty(n));

			} else if (n.startsWith("addExtension.")) {
				addExtension.put(n.substring("addExtension.".length()), p.getProperty(n));

			} else if (n.startsWith("changeExtension.")) {
				changeExtension.put(n.substring("changeExtension.".length()), p.getProperty(n));
			}
		}

		manuallySetOutputDirectory = Boolean.parseBoolean(p.getProperty("manuallySetOutputDirectory", "false"));
		appendIfAbsent = Boolean.parseBoolean(p.getProperty("appendIfAbsent", "true"));
		dontWarnAboutInPlace = Boolean.parseBoolean(p.getProperty("dontWarnAboutInPlace", "false"));

		try {
			radioChoice = Integer.parseInt(p.getProperty("radioChoice", String.valueOf(RADIO_ADD_SUB)));
		} catch (NumberFormatException e) {
			radioChoice = RADIO_ADD_SUB;
		}
	}

	public void setDefaults() {
		encIn = "Windows";
		encOut = "UTF-8";
		manuallySetOutputDirectory = false;
		appendIfAbsent = true;
		radioChoice = RADIO_ADD_SUB;
		dontWarnAboutInPlace = false;
		for (String[] pair : new String[][] {
			{ "Windows", ".win" },
			{ "windows-1250", ".win" },
			{ "Mac", ".mac" },
			{ "x-MacRoman", ".mac" },
			{ "UTF-8", ".utf8" },
			{ "Unicode", ".uni" },
			{ "UTF-16", ".uni" },
			{ "UTF-16BE", ".uni" },
			{ "UTF-16LE", ".uni" },
			{ "US-ASCII", ".asc" }
		}) {
			addSubExtension.put(pair[0], pair[1]);
			addExtension.put(pair[0], pair[1]);
			changeExtension.put(pair[0], pair[1]);
		}
	}

	public void trySave() {
		try {
			save();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void save() throws IOException {
		Properties p = new Properties();
		p.setProperty("encIn", encIn);
		p.setProperty("encOut", encOut);
		p.setProperty("inputDir", inputDir);
		if (outputDir != null) p.setProperty("outputDir", outputDir);
		p.setProperty("manuallySetOutputDirectory", String.valueOf(manuallySetOutputDirectory));
		p.setProperty("radioChoice", String.valueOf(radioChoice));
		p.setProperty("appendIfAbsent", String.valueOf(appendIfAbsent));
		if (regexIn != null) p.setProperty("regexIn", regexIn);
		if (regexOut != null) p.setProperty("regexOut", regexOut);
		if (newFile != null) p.setProperty("newFile", newFile);
		p.setProperty("dontWarnAboutInPlace", String.valueOf(dontWarnAboutInPlace));

		for (Map.Entry<String,String> ent : addSubExtension.entrySet()) {
			p.setProperty("addSubExtension." + ent.getKey(), ent.getValue());
		}
		for (Map.Entry<String,String> ent : addExtension.entrySet()) {
			p.setProperty("addExtension." + ent.getKey(), ent.getValue());
		}
		for (Map.Entry<String,String> ent : changeExtension.entrySet()) {
			p.setProperty("changeExtension." + ent.getKey(), ent.getValue());
		}

		p.save(new FileOutputStream(getFile()), "");
	}

	File getFile() {
		return new File(new File(System.getProperty("user.home")), ".encc");
	}

	public static final int RADIO_ADD_SUB = 1;
	public static final int RADIO_ADD = 2;
	public static final int RADIO_CHANGE = 3;
	public static final int RADIO_NEW_FILE = 4;
	public static final int RADIO_REGEX = 5;
	public static final int RADIO_IN_PLACE = 6;

	public String encIn;
	public String encOut;
	public String inputDir;
	public String outputDir;
	public boolean manuallySetOutputDirectory;
	public int radioChoice;
	public boolean appendIfAbsent;
	public String regexIn;
	public String regexOut;
	public Map<String,String> addSubExtension = new HashMap();
	public Map<String,String> addExtension = new HashMap();
	public Map<String,String> changeExtension = new HashMap();
	public String newFile;
	public boolean dontWarnAboutInPlace;

}
