/*
 * Copyright (c) 2010 Paul Jungwirth.
 * Licensed under the Apache License, Version 2.0:
 * http://www.apache.org/licenses/LICENSE-2.0.html
 */

package com.illuminated.encc;

import java.io.*;
import java.util.*;

/**
 * This class does the same thing as Sun's native2ascii(1) tool, except it lets
 * you specify the encoding on output as well as input and it doesn't write
 * literal "\\uxxxx"; it writes the stuff in binary like it's supposed to. See
 * the native2ascii(1) man page for a list of valid encodings. This class is not
 * really intended to be used in other code; it's just a command line utility.
 * That's fine, I think, since all but about five lines consist of processing
 * command-line arguments.
 */
public class EncFilter {

	private static final int DEFAULT_BUFFER_SIZE = 1024;
	private Set<String> _crippled = new HashSet<String>();

	public EncFilter() {
		_crippled.add("UTF-8");
		_crippled.add("UTF-16");
		_crippled.add("UTF-16BE");
		_crippled.add("UTF-16LE");
		_crippled.add("Unicode");
	}
	
	/**
	 * Interprets in according to encIn, and converts it to encOut,
	 * writing to out. Allocates buffer for the buffer size.
	 * 
	 * @param encIn The input encoding.
	 * @param encOut The output encoding.
	 * @param in The data to convert.
	 * @param out Where to send the converted data.
	 * @param buffer The size of the buffer or 0 for the default.
	 *
	 * @throws IOException
	 */
	public void run(String encIn, String encOut, InputStream in, OutputStream out, int buffer) throws IOException, CrippledException {
		Reader r = null;
		Writer w = null;
		int	len;
		char[]	b;

		if (_crippled.contains(encIn)) {
			try {
				Class.forName("com.illuminated.encc.Verify");
			} catch (ClassNotFoundException e) {
				throw new CrippledException(encIn);
			}
		}

		if (_crippled.contains(encOut)) {
			try {
				Class.forName("com.illuminated.encc.Verify");
			} catch (ClassNotFoundException e) {
				throw new CrippledException(encOut);
			}
		}

		try {
			if (buffer > 0) {
				r = new BufferedReader(new InputStreamReader(in, encIn), buffer);
				w = new BufferedWriter(new OutputStreamWriter(out, encOut), buffer);
			} else {
				r = new BufferedReader(new InputStreamReader(in, encIn));
				w = new BufferedWriter(new OutputStreamWriter(out, encOut));
				buffer = DEFAULT_BUFFER_SIZE;
			}
			b = new char[buffer];

			while ((len = r.read(b, 0, buffer)) != -1) {
				w.write(b, 0, len);
			}
		} finally {
			try {
				if (r != null) r.close();
			} finally {
				if (w != null) w.close();
			}
		}
	}

}
